=== modified file 'NEWS'
--- NEWS	2007-01-17 14:48:46 +0000
+++ NEWS	2007-01-17 15:40:08 +0000
@@ -10,6 +10,10 @@
 
   INTERNALS:
 
+    * Reserved ids (any revision-id ending in a colon) are rejected by
+      versionedfiles, repositories, branches, and working trees
+      (Aaron Bentley)
+
   BUGFIXES:
 
   TESTING:

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-12-07 16:21:39 +0000
+++ bzrlib/branch.py	2007-01-17 15:37:08 +0000
@@ -1077,6 +1077,7 @@
     def append_revision(self, *revision_ids):
         """See Branch.append_revision."""
         for revision_id in revision_ids:
+            _mod_revision.check_not_reserved_id(revision_id)
             mutter("add {%s} to revision-history" % revision_id)
         rev_history = self.revision_history()
         rev_history.extend(revision_ids)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-01-04 20:59:46 +0000
+++ bzrlib/errors.py	2007-01-17 15:37:08 +0000
@@ -199,6 +199,12 @@
         self.revision_id = revision_id
         self.branch = branch
 
+class ReservedId(BzrError):
+
+    _fmt = "Reserved revision-id {%(revision_id)s}"
+
+    def __init__(self, revision_id):
+        self.revision_id = revision_id
 
 class NoSuchId(BzrError):
 

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2006-12-21 20:07:34 +0000
+++ bzrlib/knit.py	2007-01-17 15:37:08 +0000
@@ -518,6 +518,7 @@
 
     def get_delta(self, version_id):
         """Get a delta for constructing version from some other version."""
+        self.check_not_reserved_id(version_id)
         if not self.has_version(version_id):
             raise RevisionNotPresent(version_id, self.filename)
         
@@ -685,6 +686,7 @@
         ### FIXME escape. RBC 20060228
         if contains_whitespace(version_id):
             raise InvalidRevisionId(version_id, self.filename)
+        self.check_not_reserved_id(version_id)
         if self.has_version(version_id):
             raise RevisionAlreadyPresent(version_id, self.filename)
         self._check_lines_not_unicode(lines)
@@ -797,6 +799,8 @@
 
     def get_line_list(self, version_ids):
         """Return the texts of listed versions as a list of strings."""
+        for version_id in version_ids:
+            self.check_not_reserved_id(version_id)
         text_map, content_map = self._get_content_maps(version_ids)
         return [text_map[v] for v in version_ids]
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-01-05 06:32:57 +0000
+++ bzrlib/repository.py	2007-01-17 15:37:08 +0000
@@ -97,6 +97,7 @@
 
         returns the sha1 of the serialized inventory.
         """
+        _mod_revision.check_not_reserved_id(revid)
         assert inv.revision_id is None or inv.revision_id == revid, \
             "Mismatch between inventory revision" \
             " id and insertion revid (%r, %r)" % (inv.revision_id, revid)
@@ -128,6 +129,7 @@
                        If supplied its signature_needed method will be used
                        to determine if a signature should be made.
         """
+        _mod_revision.check_not_reserved_id(rev_id)
         if config is not None and config.signature_needed():
             if inv is None:
                 inv = self.get_inventory(rev_id)

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-12-15 18:16:34 +0000
+++ bzrlib/revision.py	2007-01-17 15:37:08 +0000
@@ -454,3 +454,17 @@
         next = best_ancestor(next)
     path.reverse()
     return path
+
+
+def is_reserved_id(revision_id):
+    """Determine whether a revision id is reserved
+
+    :return: True if the revision is is reserved, False otherwise
+    """
+    return isinstance(revision_id, basestring) and revision_id.endswith(':')
+
+
+def check_not_reserved_id(revision_id):
+    """Raise ReservedId if the supplied revision_id is reserved"""
+    if is_reserved_id(revision_id):
+        raise errors.ReservedId(revision_id)

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2006-12-07 16:21:39 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-01-11 17:20:25 +0000
@@ -87,6 +87,7 @@
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
+        self.assertRaises(errors.ReservedId, br.append_revision, 'current:')
 
     def test_fetch_revisions(self):
         """Test fetch-revision operation."""

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2007-01-05 17:12:03 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2007-01-17 15:37:08 +0000
@@ -516,6 +516,13 @@
         self.assertEqual({}, graph.get_ancestors())
         self.assertEqual({}, graph.get_descendants())
 
+    def test_reserved_id(self):
+        repo = self.make_repository('repository')
+        self.assertRaises(errors.ReservedId, repo.add_inventory, 'reserved:',
+                          None, None)
+        self.assertRaises(errors.ReservedId, repo.add_revision, 'reserved:',
+                          None)
+
 
 class TestCaseWithCorruptRepository(TestCaseWithRepository):
 

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2006-10-05 05:37:25 +0000
+++ bzrlib/tests/test_revision.py	2007-01-17 15:37:08 +0000
@@ -296,3 +296,16 @@
         self.assertEqual({'B':['A'],
                           'A':[]},
                          source.get_revision_graph('B'))
+
+
+class TestReservedId(TestCase):
+
+    def test_is_reserved_id(self):
+        self.assertEqual(True, revision.is_reserved_id(NULL_REVISION))
+        self.assertEqual(True, revision.is_reserved_id(
+            revision.CURRENT_REVISION))
+        self.assertEqual(True, revision.is_reserved_id('arch:'))
+        self.assertEqual(False, revision.is_reserved_id('null'))
+        self.assertEqual(False, revision.is_reserved_id(
+            'arch:a@example.com/c--b--v--r'))
+        self.assertEqual(False, revision.is_reserved_id(None))

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2006-11-28 01:04:15 +0000
+++ bzrlib/tests/test_versionedfile.py	2007-01-11 17:20:25 +0000
@@ -140,6 +140,21 @@
         except NotImplementedError:
             pass
 
+    def test_add_reserved(self):
+        vf = self.get_file()
+        self.assertRaises(errors.ReservedId,
+            vf.add_lines, 'a:', [], ['a\n', 'b\n', 'c\n'])
+
+        self.assertRaises(errors.ReservedId,
+            vf.add_delta, 'a:', [], None, 'sha1', False, ((0, 0, 0, []),))
+
+    def test_get_reserved(self):
+        vf = self.get_file()
+        self.assertRaises(errors.ReservedId, vf.get_delta, 'b:')
+        self.assertRaises(errors.ReservedId, vf.get_texts, ['b:'])
+        self.assertRaises(errors.ReservedId, vf.get_lines, 'b:')
+        self.assertRaises(errors.ReservedId, vf.get_text, 'b:')
+
     def test_get_delta(self):
         f = self.get_file()
         sha1s = self._setup_for_deltas(f)

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-09-10 19:04:48 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-01-11 17:20:25 +0000
@@ -288,6 +288,7 @@
         wt.branch.set_revision_history(['A', 'B'])
         wt.set_last_revision('A')
         self.assertEqual(['A'], wt.get_parent_ids())
+        self.assertRaises(errors.ReservedId, wt.set_last_revision, 'A:')
 
     def test_set_last_revision_different_to_branch(self):
         # working tree formats from the meta-dir format and newer support

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2006-10-16 01:25:46 +0000
+++ bzrlib/versionedfile.py	2007-01-17 15:37:08 +0000
@@ -27,6 +27,7 @@
 from bzrlib import (
     errors,
     tsort,
+    revision,
     ui,
     )
 from bzrlib.transport.memory import MemoryTransport
@@ -58,6 +59,10 @@
         self.finished = False
         self._access_mode = access_mode
 
+    @staticmethod
+    def check_not_reserved_id(version_id):
+        revision.check_not_reserved_id(version_id)
+
     def copy_to(self, name, transport):
         """Copy this versioned file to name on transport."""
         raise NotImplementedError(self.copy_to)

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2006-09-25 14:52:38 +0000
+++ bzrlib/weave.py	2007-01-17 15:37:08 +0000
@@ -247,6 +247,7 @@
 
     def _lookup(self, name):
         """Convert symbolic version name to index."""
+        self.check_not_reserved_id(name)
         try:
             return self._name_map[name]
         except KeyError:
@@ -1120,6 +1121,7 @@
 
     def _add_lines(self, version_id, parents, lines, parent_texts):
         """Add a version and save the weave."""
+        self.check_not_reserved_id(version_id)
         result = super(WeaveFile, self)._add_lines(version_id, parents, lines,
                                                    parent_texts)
         self._save()

